`timescale 1ns/1ps

`include "if_wb_master.svh"
`include "if_wb_slave.svh"
import wishbone_pkg::*;


`define WIRE_WRF_SRC(dst, src) \
assign dst``_o.cyc = src.cyc; \
assign dst``_o.stb = src.stb; \
assign dst``_o.adr = src.adr; \
assign dst``_o.dat = src.dat_o; \
assign dst``_o.sel = src.sel; \
assign dst``_o.we  = src.we; \
assign src.ack = dst``_i.ack; \
assign src.dat_i = dst``_i.dat; \
assign src.err = dst``_i.err; \
assign src.stall = dst``_i.stall;


module main;

	reg clk_ref = 1'b0;
	reg rst_n		= 1'b0;
	reg pps			= 1'b0;
	reg pps_valid	= 1'b0;

	IWishboneMaster #(32,32) sv_wbm (clk_ref, rst_n);
	t_wishbone_master_out wbm_o;
	t_wishbone_master_in  wbm_i;
	`WIRE_WRF_SRC(wbm, sv_wbm);

  always #8ns clk_ref <= ~clk_ref;
  initial begin
    repeat(3) @(posedge clk_ref);
    rst_n <= 1'b1;
		//#90us
		//rst_n <= 1'b0;
    //repeat(3) @(posedge clk_ref);
		//rst_n <= 1'b1;
  end

	initial begin
		pps_valid <= 1'b0;
		#20us
		pps_valid <= 1'b1;
	end
	always begin
		#39us;
		#984ns;
		pps <= 1'b1;
		#16ns;
		pps <= 1'b0;
	end 

	xwrsw_gen_10mhz DUT
	(
		.rst_n_i(rst_n),
		.clk_i(clk_ref),
		.pps_i(pps),
		.pps_valid_i(pps_valid),
		.slave_i(wbm_o),
		.slave_o(wbm_i)
	);

	initial begin
		CWishboneAccessor acc;
		acc = sv_wbm.get_accessor();
		acc.set_mode(PIPELINED);
		#50us;
		acc.write('h0, 20);
		#10us;
		acc.write('h4, 30);
		#10us;
		acc.write('h8, 5);
		#30us;
		acc.write('hc, 'h0a);
		#100us;
		acc.write('hc, 'h1f);
	end

endmodule
